<?php

namespace FiloBlu\Flow\Model\From\Entity;

abstract class AbstractEntity {

    protected $specialAttributeRex = '^_';
    protected $_entityTypeCode = null;
    protected $_entityTypeModel = null;
    protected $_entityIdentifier = null;
    protected $_attributeCode = null;
    protected $_attributeValue = null;
    protected $_storeId = null;
    protected $_attributeModel = null;
    protected $_objectEntity = null;
    protected $_objectEntityId = null;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
        ) {
        $this->helper_product = $helper_product;
        $this->_storeId = $data['storeId'];
        $this->_attributeCode = trim($data['attributeCode']);
        $this->_attributeValue = trim($data['attributeValue']);
        $this->_entityIdentifier = $data['entityIdentifier'];
        $this->_entityTypeCode = $data['entityTypeCode'];
        $this->getEntityModel();
    }

    public function getObjectId() {
        if (is_null($this->_objectEntityId)) {
            $this->_objectEntityId = $this->_entityIdentifier;
        }
        return $this->_objectEntityId;
    }

    protected function getObject() {
        if (is_null($this->_objectEntity)) {
            if (!$this->getObjectId()) {
                throw new \Exception('No id found for model ' . $this->getEntityModel() . ' with given identifier ' . $this->_entityIdentifier);
            }
            $this->_objectEntity = \Magento\Framework\App\ObjectManager::getInstance()->
                get('Magento\Catalog\Model\Product')->load($this->getObjectId());
//            create($this->getEntityModel())->getById($this->getObjectId());
            //Mage::getModel($this->getEntityModel())->load($this->getObjectId());
            $this->_objectEntity->setStoreId($this->_storeId);
            if (!$this->_objectEntity->getId()) {
                throw new \Exception('No instance found for model' . $this->getEntityModel() . ' with id ' . $this->getObjectId());
            }
        }
        return $this->_objectEntity;
    }

    public function process() {
        if ($this->isSpecialFunction($this->_attributeCode)) {
            $result = $this->processSpecialFunction();
            if (is_string($result)) {
                throw new \Exception($result);
            }
        } else {
            $this->setObjectAttributeValue();
        }
    }

    protected function setObjectAttributeValue() {
        $object = $this->getObject();
        $attributeValue = $this->getAttributeValue();
        $input_type = strtolower($this->getAttribute()->getFrontend()->getInputType());
        try{
            if ($input_type === 'multiselect' && empty($attributeValue)) {
                $object->unsetData($this->_attributeCode);
                $object->save();
            } else {
                $object->setDataUsingMethod($this->_attributeCode, $attributeValue);
                $object->getResource()->saveAttribute($object, $this->_attributeCode);
            }

        }catch (\Exception $e){
            $message = "Problem while processing eavs, <br/> store id : ".$this->_storeId."<br/> id : "
                .$object->getId(). "<br/> attribute data : ".$attributeValue;
            $message .= "<br/> \Exception : ".$e->getMessage();
            $message .= "<br/> Backtrace : <br/> ".nl2br($e->getTraceAsString());
            FiloBlu_Flow_Model_From_Eavs::sendErrorEmail($message);
            throw new \Exception($e->getMessage());
        }
    }

    protected function isSpecialFunction($attribute) {
        return preg_match('/' . $this->specialAttributeRex . '/', $attribute);
    }

    protected function getSpecialActionClassName(){
        $className = "\FiloBlu\Flow\Model\Actions\From" . "\\" . ucfirst(str_replace("_","",$this->_attributeCode));

        return $className;
    }

    protected function processSpecialFunction() {

        $className = $this->getSpecialActionClassName();
        //Mage::getSingleton('fbflow/specialclassfactory')->getSpecialClassName($this);

        if(class_exists($className)){
            $specificSpecialClass = \Magento\Framework\App\ObjectManager::getInstance()->get($className);
        }
        else{
//            throw new \Exception("A special class for attribute code (" . $this->_attributeCode . ") was not found");
        }

//        if (!defined("$specificSpecialClass::ENTITY_TYPE_CODE")) {
//            throw new \Exception("Missing constant ENTITY_TYPE_CODE in $className");
//        }

        if (!method_exists($specificSpecialClass, 'process')) {
            throw new \Exception('Missing special function for attribute ' . $this->_attributeCode);
        }

//        if ($specificSpecialClass::ENTITY_TYPE_CODE != $this->getEntityTypeCode()) {
//            $message = sprintf("Object entity type defined for %s is '%s', '%s' was given.", $className, $specificSpecialClass::ENTITY_TYPE_CODE, $this->getEntityTypeCode());
//            throw new \Exception($message);
//        }

        return $specificSpecialClass->process($this->getObject(), $this->_attributeValue, $this->_storeId);
    }

    protected function getAttribute() {
        if (empty($this->_attributeModel)) {
            $this->_attributeModel = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Eav\Model\Entity\Attribute')->loadByCode($this->getEntityTypeCode(), $this->getAttributeCode());
            //Mage::getModel('eav/entity_attribute')->loadByCode($this->getEntityTypeCode(), $this->getAttributeCode());
            if (!$this->_attributeModel->getId()) {
                throw new \Exception('No attribute found for code ' . $this->_attributeCode);
            }
        }
        return $this->_attributeModel;
    }

    protected function getAttributeValue() {
        $attribute_model = $this->getAttribute();
        $attributeValue = $this->_attributeValue;
        $input_type = strtolower($attribute_model->getFrontend()->getInputType());
        if (strlen($attributeValue) > 0 && ($input_type === 'select' || $input_type === 'multiselect') && $this->getAttributeCode()!='visibility') {
            $attributeValueLabels = array();
            $attributeValueArray = array();
            if ($input_type === 'multiselect') {
                $attributeValueLabels = explode(',', $attributeValue);
                $attributeValueLabels = array_map('trim', $attributeValueLabels);
                $attributeValueLabels = array_unique($attributeValueLabels);
            } else {
                $attributeValueLabels[] = $attributeValue;
            }
            foreach ($attributeValueLabels as $attributeValueLabel) {
                $optionId = $this->getAttributeOptionValue($attributeValueLabel);
                if (empty($optionId) && $this->_storeId == 0) { // && Mage::getStoreConfig('fbflow/new_options_status') == 1) {
                    $this->addAttributeOptionValue($attributeValueLabel);
                    $optionId = $this->getAttributeOptionValue($attributeValueLabel);
                }
                if (empty($optionId)) {
                    throw new \Exception('Couldn\'t find nor add option ' . $attributeValueLabel . ' for attribute ' . $this->_attributeCode . ' entity type ' . $this->getEntityTypeCode() . ' on store ' . $this->_storeId);
                }
                $attributeValueArray[] = $optionId;
            }
            $attributeValue = implode(',', $attributeValueArray);
        }

        return $attributeValue;
    }

    protected function addAttributeOptionValue($attributeValueLabel) {
        try {

            $optionLabel = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Eav\Model\Entity\Attribute\OptionLabel');
            $optionLabel->setStoreId(0);
            $optionLabel->setLabel($attributeValueLabel);

            $option = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Eav\Model\Entity\Attribute\Option');
            $option->setLabel($optionLabel);
            $option->setStoreLabels([$optionLabel]);
            $option->setSortOrder(0);
            $option->setIsDefault(false);

            \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Eav\Api\AttributeOptionManagementInterface')
            ->add(
                \Magento\Catalog\Model\Product::ENTITY,
                $this->getAttribute()->getId(),
                $option
            );

//            $arr = array();
//            $arr['attribute_id'] = $this->getAttribute()->getId();
//            $arr['value']=array();
//            $arr['value']['option_name']=array();
//            $arr['value']['option_name'][0] = $attributeValueLabel;
//
//            $setup = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Eav\Setup\EavSetupFactory');
//            $setup->addAttributeOption($arr);

//            $setup = new Mage_Eav_Model_Entity_Setup('core_setup');
//            $setup->addAttributeOption($arr);
        } catch (\Exception $e) {
            throw new \Exception('Couldn\'t add option ' . $attributeValueLabel . ' for attribute ' . $this->_attributeCode . ' entity type ' . $this->getEntityTypeCode());
//            Mage::logException($e);
        }
    }

    protected function getAttributeOptionValue($attributeValueLabel) {

        $attribute = \Magento\Framework\App\ObjectManager::getInstance()
            ->create('\Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\CollectionFactory')
            ->create()
            ->join('eav_attribute', 'eav_attribute.attribute_id=main_table.attribute_id', 'attribute_code')
            ->join('eav_attribute_option_value', 'eav_attribute_option_value.option_id=main_table.option_id', 'value')
            ->addFieldToFilter('eav_attribute_option_value.value', array('eq' => $attributeValueLabel))
            ->addFieldToFilter('attribute_code', array('eq' => $this->_attributeCode))
            ->getFirstItem()

//        $attribute = Mage::getModel('eav/entity_attribute_option')
//            ->getCollection()
//            ->setStoreFilter()
//            ->join('attribute', 'attribute.attribute_id=main_table.attribute_id', 'attribute_code')
//            ->addFieldToFilter('tdv.value', array('eq' => $attributeValueLabel))
//            ->addFieldToFilter('attribute_code', array('eq' => $this->_attributeCode))
//            ->getFirstItem();
        ;

        return $attribute->getOptionId();
    }

    protected function getEntityModel() {
        if (is_null($this->_entityTypeModel)) {
            $entityType = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Eav\Model\Entity\Type')->loadByCode($this->getEntityTypeCode());
            if (!$entityType->getId()) {
                throw new \Exception('EntityType not found for code ' . $this->_entityTypeCode);
            }

            $this->_entityTypeModel = $entityType->getEntityModel();
        }

        return $this->_entityTypeModel;
    }

    public function getEntityTypeCode() {
        if (empty($this->_entityTypeCode)) {
            throw new \Exception('EntityTypeCode was not defined for this class');
        }

        return $this->_entityTypeCode;
    }

    public function getAttributeCode() {
        if (empty($this->_attributeCode)) {
            throw new \Exception('AttributeCode was not defined for this class');
        }

        if ($this->_attributeCode=='sku') {
            //MAGE-440 Controllare che se l'attributo è sku venga skippato
//            throw new \Exception('Sku attribute cannot be set',2 /*FiloBlu_Flow_Model_From_Eavs::SKU_ATTRIBUTE_ERR_CODE*/);
        }

        return $this->_attributeCode;
    }

}

//End of class
