<?php

namespace FiloBlu\Flow\Model\From;

class Stock extends AbstractFrom {
    
    /**
    * @var Monolog\Logger
    */
    protected $_logger;
    protected $_helper_product;
    protected $_helper_flow;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\Db $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->_helper_product = $helper_product;
        $this->_helper_flow = $helper_flow;
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Stock');
    } 

     public function process(){

        $model = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product');
        $product_id = $model->getIdBySku( $this->getSku() );

        if(!$product_id){
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(true, $message);
        }

        $model->setId($product_id);

        try {

            $stock = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\CatalogInventory\Api\StockRegistryInterface')->getStockItem($product_id);

            $qty = $this->getData('fisico_disponibile');

            $stock->setQty($qty);

            if ($qty > $stock->getMinQty()){
                $stock->setIsInStock(true);
            }
            elseif($stock->getBackorders() != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO && $stock->getMinQty() == 0) {
                $stock->setIsInStock(true);
            }
            else {
                $stock->setIsInStock(false);
            }

            $stock->save();
            
        }catch(\Exception $e){
            throw new Exception('Errore nel salvataggio dello Stock');
        }

        return $this;
    }

   


}